﻿using DataSharing.RawData;
using DataSharing.SendTask;
using DataSharing.Share.Dtos.Province;
using DataSharing.Share.Dtos.Province.XieTong;
using DataSharing.Share.Dtos.Province.XieTong.Receive;
using DataSharing.Share.Enums;
using DotNetCore.CAP;
using MapsterMapper;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using System.Net;
using XF.Domain.Dependency;
using XF.Domain.Repository;

namespace DataSharing.Province
{
    public class PusherProviderService : IPusherProviderService, IScopeDependency
    {
        #region 注入
        private readonly ILogger<PusherProviderService> _logger;
        private readonly IChannelConfigurationManager _channelConfigurationManager;
        private readonly IHttpClientFactory _httpClientFactory;
        private readonly XieTongClient _xieTongClient;
        private readonly HuiJuClient _huiJuClient;
        private readonly IRepository<DsSendTask> _dsSendTaskRepository;
        private readonly IRepository<DsSendTaskInfo> _dsSendTaskInfoRepository;
        private readonly IRepository<DsOrder> _dataOrderRepository;
        private readonly IRepository<DsTelCall> _dsTelCallRepository;

        /// <summary>
        /// 
        /// </summary>
        /// <param name="logger"></param>
        /// <param name="channelConfigurationManager"></param>
        /// <param name="httpClientFactory"></param>
        /// <param name="xieTongClient"></param>
        /// <param name="huiJuClient"></param>
        /// <param name="dsSendTaskRepository"></param>
        /// <param name="dsSendTaskInfoRepository"></param>
        /// <param name="dataOrderRepository"></param>
        /// <param name="dsTelCallRepository"></param>
        public PusherProviderService(ILogger<PusherProviderService> logger,
            IChannelConfigurationManager channelConfigurationManager,
            IHttpClientFactory httpClientFactory,
            XieTongClient xieTongClient,
            HuiJuClient huiJuClient,
            IRepository<DsSendTask> dsSendTaskRepository,
            IRepository<DsSendTaskInfo> dsSendTaskInfoRepository,
            IRepository<DsOrder> dataOrderRepository,
            IRepository<DsTelCall> dsTelCallRepository)
        {
            _logger = logger;
            _channelConfigurationManager = channelConfigurationManager;
            _httpClientFactory = httpClientFactory;
            _xieTongClient = xieTongClient;
            _huiJuClient = huiJuClient;
            _dsSendTaskRepository = dsSendTaskRepository;
            _dsSendTaskInfoRepository = dsSendTaskInfoRepository;
            _dataOrderRepository = dataOrderRepository;
            _dsTelCallRepository = dsTelCallRepository;
        }
        #endregion

        /// <summary>
        /// 省平台推送数据
        /// </summary>
        /// <param name="dto"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        public async Task SendProvinceDataPusher(DsSendTask dto, CancellationToken cancellationToken)
        {
            if (dto is null)
                return;

            List<string> list = new()
            {
            "RGDH9951150024050100003",
            "RGDH9951150024050100002",
            "RGDH9951150024050400002",
            "RGDH9951150024050400004",
            "RGDH9951150024050400005",
            "RGDH9951150024050400009",
            "RGDH9951150024050400010",
            "RGDH9951150024050400011",
            "RGDH9951150024050400008",
            "RGDH9951150024050500001",
            "RGDH9951150024050500005",
            "RGDH9951150024050500009",
            "RGDH9951150024050500004",
            "RGDH9951150024050600001",
            "RGDH9951150024050600002",
            "RGDH9951150024050600006",
            "RGDH9951150024050600005",
            "RGDH9951150024050600008",
            "RGDH9951150024050600004",
            "RGDH9951150024050600010",
            "RGDH9951150024050600011",
            "RGDH9951150024050700001",
            "RGDH9951150024050800001",
            "RGDH9951150024050800005",
            "RGDH9951150024050800009",
            "RGDH9951150024050800008",
            "RGDH9951150024050800012",
            "RGDH9951150024050800013",
            "RGDH9951150024050800007",
            "RGDH9951150024050900002",
            "RGDH9951150024050900001",
            "RGDH9951150024050900006",
            "RGDH9951150024050900009",
            "RGDH9951150024050900007",
            "RGDH9951150024051000001",
            "RGDH9951150024051100001",
            "RGDH9951150024050100001",
            "RGDH9951150024051100004",
            "RGDH9951150024051100006",
            "RGDH9951150024051200002",
            "RGDH9951150024051200004",
            "RGDH9951150024051300002",
            "RGDH9951150024051300001",
            "RGDH9951150024051400001",
            "RGDH9951150024051500001",
            "RGDH9951150024051500002",
            "RGDH9951150024051500006",
            "RGDH9951150024051500003",
            "RGDH9951150024041200003",
            "RGDH9951150024041200004",
            "RGDH9951150024041200005",
            "RGDH9951150024041300001",
            "RGDH9951150024041300005",
            "RGDH9951150024041300006",
            "RGDH9951150024041300014",
            "RGDH9951150024041300019",
            "RGDH9951150024041300011",
            "RGDH9951150024041300018",
            "RGDH9951150024041300021",
            "RGDH9951150024041300022",
            "RGDH9951150024041300024",
            "RGDH9951150024041400001",
            "RGDH9951150024041400002",
            "RGDH9951150024041400003",
            "RGDH9951150024041400006",
            "RGDH9951150024041500004",
            "RGDH9951150024041500001",
            "RGDH9951150024041500007",
            "RGDH9951150024041500009",
            "RGDH9951150024041500008",
            "RGDH9951150024041500010",
            "RGDH9951150024041500006",
            "RGDH9951150024041600001",
            "RGDH9951150024041700002",
            "RGDH9951150024041700006",
            "RGDH9951150024041700003",
            "RGDH9951150024041700001",
            "RGDH9951150024041700005",
            "RGDH9951150024041800003",
            "RGDH9951150024041800005",
            "RGDH9951150024041800002",
            "RGDH9951150024041800001",
            "RGDH9951150024041800015",
            "RGDH9951150024041800008",
            "RGDH9951150024041800018",
            "RGDH9951150024041800022",
            "RGDH9951150024041800031",
            "RGDH9951150024041800030",
            "RGDH9951150024032900004",
            "RGDH9951150024032900001",
            "RGDH9951150024032900002",
            "RGDH9951150024032900003",
            "RGDH9951150024032900008",
            "RGDH9951150024032900010",
            "RGDH9951150024032900012",
            "RGDH9951150024032900015",
            "RGDH9951150024032900014",
            "RGDH9951150024032900019",
            "RGDH9951150024032900018",
            "RGDH9951150024032900017",
            "RGDH9951150024032900013",
            "RGDH9951150024032900022",
            "RGDH9951150024032900023",
            "RGDH9951150024032900025",
            "RGDH9951150024033000002",
            "RGDH9951150024033000004",
            "RGDH9951150024033000003",
            "RGDH9951150024033000005",
            "RGDH9951150024033100006",
            "RGDH9951150024033100005",
            "RGDH9951150024033100003",
            "RGDH9951150024033100004",
            "RGDH9951150024033100009",
            "RGDH9951150024033100002",
            "RGDH9951150024033100013",
            "RGDH9951150024033100011",
            "RGDH9951150024033100014",
            "RGDH9951150024033100015",
            "RGDH9951150024033100017",
            "QT9951150024040100006",
            "RGDH9951150024040100007",
            "RGDH9951150024040100003",
            "RGDH9951150024040100009",
            "RGDH9951150024040100002",
            "RGDH9951150024040100010",
            "RGDH9951150024040100014",
            "RGDH9951150024040100012",
            "RGDH9951150024040100017",
            "RGDH9951150024040100015",
            "RGDH9951150024040200001",
            "RGDH9951150024040200006",
            "RGDH9951150024040200004",
            "RGDH9951150024040200007",
            "RGDH9951150024040200009",
            "RGDH9951150024040200012",
            "RGDH9951150024040200015",
            "RGDH9951150024040200014",
            "RGDH9951150024040200019",
            "RGDH9951150024041900001",
            "RGDH9951150024041900003",
            "RGDH9951150024041900009",
            "RGDH9951150024042000001",
            "RGDH9951150024042100002",
            "RGDH9951150024042100003",
            "RGDH9951150024042100004",
            "RGDH9951150024042200002",
            "RGDH9951150024042200005",
            "RGDH9951150024042200003",
            "RGDH9951150024042200008",
            "RGDH9951150024042200006",
            "RGDH9951150024042200011",
            "RGDH9951150024042200012",
            "RGDH9951150024042200015",
            "RGDH9951150024042200016",
            "RGDH9951150024042200014",
            "RGDH9951150024042200021",
            "RGDH9951150024042200023",
            "RGDH9951150024042200025",
            "RGDH9951150024040200018",
            "RGDH9951150024040200022",
            "RGDH9951150024040200023",
            "RGDH9951150024040300001",
            "RGDH9951150024040300003",
            "RGDH9951150024042200027",
            "RGDH9951150024042300002",
            "RGDH9951150024042300001",
            "RGDH9951150024042300004",
            "RGDH9951150024042300006",
            "RGDH9951150024040300007",
            "RGDH9951150024040300009",
            "RGDH9951150024040300010",
            "RGDH9951150024040300012",
            "RGDH9951150024040400001",
            "RGDH9951150024042300007",
            "RGDH9951150024042400001",
            "RGDH9951150024042400005",
            "RGDH9951150024042400011",
            "RGDH9951150024042400013",
            "RGDH9951150024040500002",
            "RGDH9951150024040500001",
            "RGDH9951150024040600001",
            "RGDH9951150024040600002",
            "RGDH9951150024040600005",
            "RGDH9951150024040600004",
            "RGDH9951150024040700001",
            "RGDH9951150024040700002",
            "RGDH9951150024040700006",
            "RGDH9951150024040700005",
            "RGDH9951150024040700007",
            "RGDH9951150024040700009",
            "RGDH9951150024040700008",
            "RGDH9951150024040700011",
            "RGDH9951150024040700012",
            "RGDH9951150024040700014",
            "RGDH9951150024040700015",
            "RGDH9951150024040700019",
            "RGDH9951150024040700017",
            "RGDH9951150024040700020",
            "RGDH9951150024040700025",
            "RGDH9951150024042400015",
            "RGDH9951150024042400014",
            "RGDH9951150024042500002",
            "RGDH9951150024042500003",
            "RGDH9951150024042600002",
            "RGDH9951150024040700026",
            "RGDH9951150024040700023",
            "RGDH9951150024040800001",
            "RGDH9951150024040800003",
            "RGDH9951150024040800005",
            "RGDH9951150024040800008",
            "RGDH9951150024040800010",
            "RGDH9951150024040800011",
            "RGDH9951150024040800012",
            "RGDH9951150024040800014",
            "RGDH9951150024040900001",
            "RGDH9951150024040900002",
            "QT9951150024040900006",
            "RGDH9951150024040900009",
            "RGDH9951150024040900008",
            "RGDH9951150024042600003",
            "RGDH9951150024042600006",
            "RGDH9951150024042800001",
            "RGDH9951150024042800003",
            "RGDH9951150024042800005",
            "RGDH9951150024041000005",
            "RGDH9951150024041000007",
            "RGDH9951150024041000006",
            "RGDH9951150024041000010",
            "RGDH9951150024041000004",
            "RGDH9951150024042800009",
            "RGDH9951150024042800006",
            "RGDH9951150024042800008",
            "RGDH9951150024042800012",
            "RGDH9951150024042900002",
            "RGDH9951150024042900001",
            "RGDH9951150024042900005",
            "RGDH9951150024043000005",
            "RGDH9951150024043000007",
            "RGDH9951150024043000008",
            "RGDH9951150024041100001",
            "RGDH9951150024041100006",
            "RGDH9951150024041100004"
        };
            for (int i = 0; i < list.Count; i++)
            {
                if (dto.Request.Contains(list[i]))
                {
                    dto.IsSuccess = true;
                    await _dsSendTaskRepository.UpdateAsync(dto, cancellationToken);
                    return;
                }
            }

            var configProvince = _channelConfigurationManager.GetConfigurationProvince();
            if (configProvince.SendTaskStartTime != null && configProvince.SendTaskStartTime > dto.GenerationTime)
            {
                dto.IsSuccess = true;
                await _dsSendTaskRepository.UpdateAsync(dto, cancellationToken);
            }
            var name = GetName(dto.Path);

            _logger.LogWarning("省请求参数：" + "--------------" + dto.Request);
            ProvinceResponse result = new();
            if (dto.PathType == EPathType.HuiJu) //汇聚推送
                result = await _huiJuClient.RequestAsync<ProvinceResponse>(configProvince.HuiJu + dto.Path, dto.HttpMethod, dto.Request, cancellationToken);

            else if (dto.PathType == EPathType.XieTong)//协同推送
                result = await _xieTongClient.RequestAsync<ProvinceResponse>(configProvince.XieTong + dto.Path, dto.HttpMethod, dto.Request, cancellationToken);

            else if (dto.PathType == EPathType.File)  //附件
            {
                if (dto != null && dto.FileJson != null && dto.FileJson.Count > 0)
                {
                    var itemFile = dto.FileJson[0];
                    result = await SendFileData(dto.Request, itemFile.FileId, itemFile.FileId, cancellationToken);
                }
            }

            _logger.LogWarning("省接口返回：" + name + "--------------" + Newtonsoft.Json.JsonConvert.SerializeObject(result));

            if (result is not null)
            {
                #region 处理推送任务表，推送明细表
                //写入明细表
                DsSendTaskInfo dsSendTaskInfo = new() { TaskId = dto.Id, Result = result };

                //看看需不需要处理一些信息
                dto.LastTime = DateTime.Now;
                dto.SendTimes = dto.SendTimes + 1;

                if (dto.FirstTime is null)
                    dto.FirstTime = DateTime.Now;

                if (result.ReturnInfo is not null && result.ReturnInfo.Code == "1")
                {
                    dto.IsSuccess = true;
                    dsSendTaskInfo.IsSuccess = true;
                }
                await _dsSendTaskInfoRepository.AddAsync(dsSendTaskInfo, cancellationToken);
                await _dsSendTaskRepository.UpdateAsync(dto, cancellationToken);
                #endregion

                #region 处理工单的推送时间，用于后面计算及时率
                if (result.ReturnInfo is not null && result.ReturnInfo.Code == "1")
                {
                    if (dto.TaskType == "SubmitCaseInfo")
                    {
                        var dataOrder = System.Text.Json.JsonSerializer.Deserialize<ProvinceListRequestSubmitCaseInfo>(dto.Request);//
                        if (dataOrder != null && dataOrder.Data != null)
                        {
                            var orderItems = dataOrder.Data.Items;
                            foreach (var item in orderItems)
                            {
                                var dsOrder = await _dataOrderRepository.GetAsync(p => p.ProvinceNo == item.CaseSerial, cancellationToken);
                                if (dsOrder != null)
                                {
                                    if (dsOrder.FirstSendProvinceTime is null)
                                        dsOrder.FirstSendProvinceTime = DateTime.Now;

                                    dsOrder.LastSendProvinceTime = DateTime.Now;
                                    await _dataOrderRepository.UpdateAsync(dsOrder);
                                }
                            }
                        }
                    }
                }
                #endregion

                #region 处理通话记录推送时间
                if (result.ReturnInfo is not null && result.ReturnInfo.Code == "1")
                {
                    if (dto.TaskType == "SubmitCaseRecord")
                    {
                        var telCall = await _dsTelCallRepository.GetAsync(p => p.TaskId == dto.Id, cancellationToken);
                        if (telCall != null)
                        {
                            if (telCall.FirstSendProvinceTime is null)
                                telCall.FirstSendProvinceTime = DateTime.Now;

                            telCall.LastSendProvinceTime = DateTime.Now;
                            await _dsTelCallRepository.UpdateAsync(telCall, cancellationToken);
                        }
                    }
                }
                #endregion
            }
        }

        /// <summary>
        /// 附件读取以及附件上传 
        /// </summary>
        /// <param name="dto"></param>
        /// <param name="fileName"></param>
        /// <param name="fileId"></param>
        /// <returns></returns>
        private async Task<ProvinceResponse> SendFileData(string request, string fileName, string fileId, CancellationToken cancellationToken)
        {
            //获取配置信息
            var configProvince = _channelConfigurationManager.GetConfigurationProvince();
            var businessFile = _channelConfigurationManager.GetConfigurationBusinessFile();
            byte[] fileContentBytes = null;
            try
            {
                var baseAddress = businessFile.BaseUrl;
                if (!baseAddress.EndsWith('/'))
                    baseAddress += "/";
                string url = string.Format("{0}{1}?Source={2}&Id={3}", baseAddress, businessFile.DownloadUrlAddress, "hotline", fileId);
                using (var client = _httpClientFactory.CreateClient())
                using (var response = await client.GetAsync(url))
                {

                    if (response.StatusCode == HttpStatusCode.OK)
                    {
                        fileName = response.Content.Headers.ContentDisposition.FileNameStar;
                        using var memoryStream = new MemoryStream();
                        await response.Content.CopyToAsync(memoryStream);
                        memoryStream.Seek(0, SeekOrigin.Begin);

                        fileContentBytes = memoryStream.ToArray();
                    }
                }
                if (fileContentBytes != null)
                {
                    //获取附件上传Token
                    string strToken = await _xieTongClient.GetTokenAsync(cancellationToken);
                    Dictionary<string, object> dicParam = new()
                    {
                        { "params",  request } // 第一个接口参数，json格式字符串
                    };

                    // 构造字典文件数据
                    // 接口参数名称为files
                    CFormUpload.FileParameter fileParameter = new CFormUpload.FileParameter("files", fileContentBytes, fileName, null);
                    dicParam.Add(fileName, fileParameter);

                    var baseAddressHuiJu = configProvince.HuiJu;
                    if (!baseAddressHuiJu.EndsWith('/'))
                        baseAddressHuiJu += "/";

                    string strUrl = baseAddressHuiJu + "get_case_material_info";
                    // 上传附件
                    string strResult = CFormUpload.MultipartFormDataPost(strUrl, null, dicParam, strToken);

                    _logger.LogWarning("省接口上传附件返回：--------------" + strResult);

                    if (!string.IsNullOrEmpty(strResult))
                        return JsonConvert.DeserializeObject<ProvinceResponse>(strResult);
                }
                return new ProvinceResponse();
            }
            catch (Exception ex)
            {
                return new ProvinceResponse();
            }
        }

        /// <summary>
        /// 获取接口名称
        /// </summary>
        /// <param name="path"></param>
        /// <returns></returns>
        private string GetName(string path)
        {
            string name = "";
            switch (path)
            {
                case "get_case_back_apply":
                    name = "业务协同-退回申请接口";
                    break;
                case "send_supervise_process_info":
                    name = "业务协同-服务工单督办过程接口";
                    break;
                case "send_supervise_result_info":
                    name = "业务协同-服务工单督办结果接口";
                    break;
                case "screen_case_info_send":
                    name = "业务协同-工单发起甄别接口";
                    break;
                case "delay_case_info_send":
                    name = "业务协同-服务工单申请延时接口";
                    break;
                case "submit_case_info":
                    name = "业务汇聚-服务工单受理接口";
                    break;
                case "submit_case_record":
                    name = "业务汇聚-电话记录";
                    break;
                case "get_case_process_receive":
                    name = "业务协同-服务工单交办处理过程接口";
                    break;
                case "submit_case_process":
                    name = "业务汇聚-服务工单处理过程接口";
                    break;
                case "submit_case_total":
                    name = "业务汇聚-统计数据接口";
                    break;
                case "supply_case_info_send":
                    name = "业务协同-上报补充诉求接口";
                    break;
                case "revoke_case_info":
                    name = "业务协同-服务工单撤单接口";
                    break;
                case "remind_case_info":
                    name = "业务协同-服务工单催单接口";
                    break;
                case "get_case_result_receive":
                    name = "业务协同-服务工单交办处理结果接口";
                    break;
                case "zmhd_case_info_public":
                    name = "业务协同-政民互动提交公开";
                    break;
                case "get_knowledge_info_send":
                    name = "业务协同-知识库上报接口";
                    break;
                case "get_knowledge_info_update":
                    name = "业务协同-知识库变更接口";
                    break;
                case "get_knowledge_info_abandon":
                    name = "业务协同-知识库下架接口";
                    break;
                case "sendrealtimestatus":
                    name = "实时服务上报接口";
                    break;
                case "get_visit_info_receive":
                    name = "业务协同-服务工单交办评价接口";
                    break;
                case "submit_visit_info":
                    name = "业务汇聚-服务工单回访评价接口";
                    break;
                case "submit_case_result":
                    name = "业务汇聚-服务工单处理结果接口";
                    break;
                case "submit_case_extends":
                    name = "业务汇聚-服务工单拓展信息接口";
                    break;
                case "send_case_info":
                    name = "业务协同-服务工单上报";
                    break;
                case "get_case_material_info":
                    name = "业务汇聚-材料接口";
                    break;
                default:
                    break;
            }

            return name;
        }
    }
}
